#include "mltwave.hpp"
#include <string.h>

/*------------------------------------------------------------------------------
| WAVE::WAVE                                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
WAVE::WAVE( IMMWaveAudio*     aWavePlayer,
            unsigned long     windowid,
            IWindow*          parent,
            IWindow*          owner,
            char*             pDir,
            unsigned long     evtId)
   : IMMPlayerPanel(windowid,parent,owner,IMMDevice::waveAudio),
     readout     (WAVEREADOUTID, this,this),
     name        (WAVENAMEID, this, this),
     loadit      (LOADID,this,this,IRectangle(),
                  IWindow::visible | IAnimatedButton::animateWhenLatched),
     saveButton  (SAVEID,this,this),
     lineIn      (WAVELINEINID,this,this),
     microphone  (WAVEMICROPHONEID,this,this),
     pWavePlayer(aWavePlayer),
     observer(0),
     handler(0),
     rMaxTime(9000)
{
   setPlayableDevice(pWavePlayer);
   iPerCt = 0;
   strcpy(pPath, pDir);
   doneEvtId = evtId;

   rec = new IAnimatedButton(RECID,this,this,IRectangle(),
                             ICustomButton::autoLatch |
                             ICustomButton::latchable |
                             IWindow::visible | IAnimatedButton::animateWhenLatched);
   saveButton.setText("Done");
   lineIn.enableAutoSelect();
   lineIn.setText("LineIn");
   microphone.enableAutoSelect();
   microphone.setText("Microphone");
   microphone.select();

   //Add the additional buttons to the player panel.
   addToCell(rec      ,     7, 1, 1, 2);
   addToCell(&lineIn   ,    8, 1, 1, 1);
   addToCell(&microphone,   8, 2, 1, 1);
   addToCell(&saveButton,   9, 1, 1, 1);
   addToCell(&loadit  ,    11, 1, 1, 2);
   addToCell(&readout ,    13, 1, 1, 2);
   addToCell(&name    ,     2, 3, 5, 1);
   rec->disable();
   playButton()->disable();
   saveButton.disable();
   rewindButton()->disable();
   fastForwardButton()->disable();

   //Put the bitmaps on the buttons.
   rec->setBitmaps(IAnimatedButton::record);
   loadit.setBitmaps(IAnimatedButton::eject);

   //Put text on the buttons.
   rec->setText("Record");
   loadit.setText("SignIn");

   //Set up the title
   name.setText("Raffle Entry Panel");
   name.setForegroundColor(IColor(IColor::red));

   //Set up the display
   IFont("LED",10).setWindowFont(&readout);
   readout.setForegroundColor(IColor(0,160,0));
   readout.setBackgroundColor(IColor(IColor::black));
   readout.setText("MIN:SEC 00:00");
   readout.setLimit(14);

   if (pWavePlayer)
   {
      handler  = new WaveHandler();
      handler->handleEventsFor(this);
      observer = new WaveObserver(*this);
      observer->handleNotificationsFor(*pWavePlayer);
   }
}


/*------------------------------------------------------------------------------
| WAVE::~WAVE                                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
WAVE::~WAVE()
{
  if (observer)
     observer->stopHandlingNotificationsFor(*pWavePlayer);
  if (handler)
     handler->stopHandlingEventsFor(this);
}

/*------------------------------------------------------------------------------
| WAVE::recordButton                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* WAVE::recordButton () const
{
  return rec;
}

/*------------------------------------------------------------------------------
| WAVE::wavePlayer                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMWaveAudio* WAVE::wavePlayer () const
{
  return pWavePlayer;
}


/*------------------------------------------------------------------------------
| WAVE::getNextName                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void WAVE::getNextName (char * pName)
{
  iPerCt++;
  IString temp = IString(pPath) +
                 "P" +
                 IString(iPerCt) +
                 ".WAV";

  strcpy(pName, temp.asString());
  return;
}

/*------------------------------------------------------------------------------
| WAVE::getCurIndex                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int  WAVE::getCurIndex ( )
{
  return iPerCt;
}

/*------------------------------------------------------------------------------
| WAVE::stopRecording                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void WAVE::stopRecording(const IMMPlayerPanel& panel)
{
  handler->stop(panel);
  return;
}


/*------------------------------------------------------------------------------
| WaveHandler::WaveHandler                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
WaveHandler::WaveHandler()
          : WaveHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| WaveHandler::command                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean WaveHandler::command(ICommandEvent& event)
{
  Boolean handled = false;
  WAVE* panel = 0;

  switch (event.commandId())
  {
    case LOADID:
        {
         char  newName[64];

         panel = (WAVE*) (event.window());
         panel->getNextName(newName);
         panel->wavePlayer()->loadOnThread(newName);
         stop(*panel);
         panel->wavePlayer()->startPositionTracking(IMMTime(3000));
         panel->recordButton()->enable();
         panel->playButton()->enable();
         panel->rewindButton()->enable();
         panel->fastForwardButton()->enable();
         panel->saveButton.enable();
         }
         handled=true;
         break;
    case SAVEID:
         panel = (WAVE*) (event.window());
         stop(*panel);
         panel->wavePlayer()->save();
         panel->recordButton()->disable();
         panel->playButton()->disable();
         panel->rewindButton()->disable();
         panel->fastForwardButton()->disable();
         panel->saveButton.disable();
         panel->parent()->postEvent(panel->doneEvtId
                                    ,IEventParameter1(panel->getCurIndex())
                                    ,IEventParameter2(0) );
         handled=true;
         break;
    case RECID:
         panel = (WAVE*) (event.window());
         panel->stopButton()->enable();
         panel->pauseButton()->disable();
         panel->playButton()->disable();
         panel->rewindButton()->disable();
         panel->fastForwardButton()->disable();
         if (panel->lineIn.isSelected())
            panel->wavePlayer()->enableConnector(IMMDevice::lineIn);
         else
            panel->wavePlayer()->enableConnector(IMMDevice::microphones);
         panel->wavePlayer()->acquire(IMMDevice::isolatedExclusive);
         panel->wavePlayer()->setVolume(95);
         panel->wavePlayer()->record();
         handled = true;
         break;
    default:
         handled = Inherited::command(event);
         break;
  } /* endswitch */
  return handled;
}

/*------------------------------------------------------------------------------
| WaveHandler::stop                                                            |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean WaveHandler::stop(const IMMPlayerPanel& panel)
{
  panel.playButton()->enable();
  panel.rewindButton()->enable();
  panel.fastForwardButton()->enable();
  Boolean retval = Inherited::stop(panel);
  panel.playableDevice()->release();
  return retval;
}

/*------------------------------------------------------------------------------
| WaveObserver::WaveObserver                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
WaveObserver::WaveObserver(WAVE& wavePanel)
          : WaveObserver::Inherited(),
          panel(wavePanel)
{}

/*------------------------------------------------------------------------------
| WaveObserver::dispatchNotificationEvent                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
WaveObserver& WaveObserver::dispatchNotificationEvent(const INotificationEvent& event)
{
   if (event.notificationId() == IMMDevice::positionChangeId)
   {
      IMMPositionChangeEvent* positionEvent = (IMMPositionChangeEvent*)(event.eventData().asUnsignedLong());

      IMMTime time(positionEvent->position());
      panel.readout.setText(IString("MIN:SEC ") +
                      IString(time.minutes()).rightJustify(2,'0') +
                      ":" +
                      IString(time.seconds()).rightJustify(2,'0'));
      if (panel.wavePlayer()->mode() == IMMDevice::recording)
        if (time.seconds() >= 3)
            panel.stopRecording(panel);
      if (!panel.playButton()->isLatched())
         if (panel.wavePlayer()->mode()==IMMDevice::playing)
            panel.playButton()->latch();
   } /* endif */
   if (event.notificationId() == IMMDevice::commandNotifyId)
   {
      IMMNotifyEvent* notifyEvent = (IMMNotifyEvent*)(event.eventData().asUnsignedLong());
      if (notifyEvent->command() == IMMNotifyEvent::play)
         panel.playButton()->unlatch();
      if (notifyEvent->command() == IMMNotifyEvent::record)
         panel.recordButton()->unlatch();
   } /* endif */
   return *this;
}
